@@ -24,7 +24,7 @@ function find_syslinux_file {
     # output argument is the full path of isolinux.bin
     local syslinux_file=""
 
-    for file in /usr/{share,lib,libexec}/*/"$1" ; do
+    for file in /usr/{share,lib,libexec,lib/syslinux}/*/"$1" ; do
         if [[ -s "$file" ]]; then
             syslinux_file="$file"
             break # for loop
@@ -47,7 +47,7 @@ function find_syslinux_modules_dir {
     if version_newer "$syslinux_version" 5.00; then
         # check for the default location - fast and easy
         if [[ -d /usr/lib/syslinux/modules ]]; then
-            if (( USING_UEFI_BOOTLOADER )); then
+            if is_true $USING_UEFI_BOOTLOADER ; then
                 syslinux_modules_dir=/usr/lib/syslinux/modules/efi64
             else
                 syslinux_modules_dir=/usr/lib/syslinux/modules/bios
@@ -59,7 +59,7 @@ function find_syslinux_modules_dir {
             file=$( find /usr -name "$1" 2>/dev/null | tail -1 )
             syslinux_modules_dir=$( dirname "$file" )        # /usr/lib/syslinux/modules/efi32
             syslinux_modules_dir=${syslinux_modules_dir%/*}  # /usr/lib/syslinux/modules
-            if (( USING_UEFI_BOOTLOADER )); then
+            if is_true $USING_UEFI_BOOTLOADER ; then
                 syslinux_modules_dir=${syslinux_modules_dir}/efi64
             else
                 syslinux_modules_dir=${syslinux_modules_dir}/bios
@@ -263,9 +263,9 @@ function make_syslinux_config {
 	if [ "$ISO_DEFAULT" == "manual" ] ; then
                echo "default rear"
                syslinux_menu "default"
-        fi 
+        fi
 	echo ""
-	
+
 	echo "say rear - Recover $(uname -n)"
 	echo "label rear-automatic"
 	syslinux_menu "label ^Automatic Recover $(uname -n)"
@@ -277,9 +277,9 @@ function make_syslinux_config {
 	if [ "$ISO_DEFAULT" == "automatic" ] ; then
                echo "default rear-automatic"
                syslinux_menu "default"
-        fi	
+        fi
 	echo ""
-	
+
 	syslinux_menu separator
 	echo "label -"
 	syslinux_menu "label Other actions"
@@ -447,10 +447,10 @@ function make_syslinux_config {
 function create_ebiso_elilo_conf {
 cat << EOF
 timeout = 5
-default = "Relax and Recover (no Secure Boot)"
+default = "Relax-and-Recover (no Secure Boot)"
 
 image = kernel
-    label = "Relax and Recover (no Secure Boot)"
+    label = "Relax-and-Recover (no Secure Boot)"
     initrd = initrd.cgz
 EOF
     [[ -n $KERNEL_CMDLINE ]] && cat << EOF
@@ -460,6 +460,9 @@ EOF
 
 # Create configuration grub
 function create_grub2_cfg {
+# SLES12 SP1 boot throw kernel panic without root= set
+root_uuid=$(mount | grep -w 'on /' | awk '{print $1}' | xargs blkid -s UUID -o value)
+
 cat << EOF
 set default="0"
 
@@ -479,16 +482,16 @@ set timeout=5
 search --no-floppy --file /boot/efiboot.img --set
 #set root=(cd0)
 
-menuentry "Relax and Recover (no Secure Boot)"  --class gnu-linux --class gnu --class os {
+menuentry "Relax-and-Recover (no Secure Boot)"  --class gnu-linux --class gnu --class os {
      echo 'Loading kernel ...'
-     linux /isolinux/kernel $KERNEL_CMDLINE
+     linux /isolinux/kernel root=UUID=$root_uuid $KERNEL_CMDLINE
      echo 'Loading initial ramdisk ...'
      initrd /isolinux/initrd.cgz
 }
 
-menuentry "Relax and Recover (Secure Boot)"  --class gnu-linux --class gnu --class os {
+menuentry "Relax-and-Recover (Secure Boot)"  --class gnu-linux --class gnu --class os {
      echo 'Loading kernel ...'
-     linuxefi /isolinux/kernel $KERNEL_CMDLINE
+     linuxefi /isolinux/kernel root=UUID=$root_uuid $KERNEL_CMDLINE
      echo 'Loading initial ramdisk ...'
      initrdefi /isolinux/initrd.cgz
 }
@@ -502,4 +505,3 @@ menuentry "Exit to EFI Shell" {
 }
 EOF
 }
-
