@@ -2,45 +2,60 @@
 #
 # shell script functions for Relax-and-Recover
 #
-# This file is part of Relax and Recover, licensed under the GNU General
-# Public License. Refer to the included LICENSE for full text of license.
+# This file is part of Relax-and-Recover, licensed under the GNU General
+# Public License. Refer to the included COPYING for full text of license.
 #
 # convert tabs into 4 spaces with: expand --tabs=4 file >new-file
 
 # source a file given in $1
-function Source() {
-    # skip empty
-    if test -z "$1" ; then
+function Source () {
+    local source_file="$1"
+    # Skip if source file name is empty:
+    if test -z "$source_file" ; then
+        Debug "Skipping Source() because it was called with empty source file name"
         return
     fi
-    [[ ! -d "$1" ]]
-    StopIfError "$1 is a directory, cannot source"
-    if test -s "$1" ; then
-        local relname="${1##$SHARE_DIR/}"
-        if test "$SIMULATE" && expr "$1" : "$SHARE_DIR" >&8; then
-            # simulate sourcing the scripts in $SHARE_DIR
-            LogPrint "Source $relname"
-        else
-            # step-by-step mode or breakpoint if needed
-            # usage of the external variable BREAKPOINT:
-            # sudo BREAKPOINT="*foo*" rear mkrescue
-            [[ "$STEPBYSTEP" || ( "$BREAKPOINT" && "$relname" == "$BREAKPOINT" ) ]] && read -p "Press ENTER to include '$1' ..." 2>&1
-
-            Log "Including ${1##$SHARE_DIR/}"
-            test "$DEBUGSCRIPTS" && set -x
-            . "$1"
-            test "$DEBUGSCRIPTS" && set +x
-            [[ "$BREAKPOINT" && "$relname" == "$BREAKPOINT" ]] && read -p "Press ENTER to continue ..." 2>&1
-        fi
-    else
-        Debug "Skipping $1 (file not found or empty)"
+    # Ensure source file is not a directory:
+    test -d "$source_file" && Error "Source file '$source_file' is a directory, cannot source"
+    # Skip if source file does not exist of if its content is empty:
+    if ! test -s "$source_file" ; then
+        Debug "Skipping Source() because source file '$source_file' not found or empty"
+        return
     fi
+    # Clip leading standard path to rear files (usually /usr/share/rear/):
+    local relname="${source_file##$SHARE_DIR/}"
+    # Simulate sourcing the scripts in $SHARE_DIR
+    if test "$SIMULATE" && expr "$source_file" : "$SHARE_DIR" >&8; then
+        LogPrint "Source $relname"
+        return
+    fi
+    # Step-by-step mode or breakpoint if needed
+    # Usage of the external variable BREAKPOINT: sudo BREAKPOINT="*foo*" rear mkrescue
+    # an empty default value is set to avoid 'set -eu' error exit if BREAKPOINT is unset:
+    : ${BREAKPOINT:=}
+    [[ "$STEPBYSTEP" || ( "$BREAKPOINT" && "$relname" == "$BREAKPOINT" ) ]] && read -p "Press ENTER to include '$source_file' ..." 2>&1
+    Log "Including $relname"
+    # DEBUGSCRIPTS mode settings:
+    if test "$DEBUGSCRIPTS" ; then
+        Debug "Entering debugscripts mode via 'set -$DEBUGSCRIPTS_ARGUMENT'."
+        local saved_bash_flags_and_options_commands="$( get_bash_flags_and_options_commands )"
+        set -$DEBUGSCRIPTS_ARGUMENT
+    fi
+    # The actual work (source the source file):
+    source "$source_file"
+    # Undo DEBUGSCRIPTS mode settings:
+    if test "$DEBUGSCRIPTS" ; then
+        Debug "Leaving debugscripts mode (back to previous bash flags and options settings)."
+        apply_bash_flags_and_options_commands "$saved_bash_flags_and_options_commands"
+    fi
+    # Breakpoint if needed:
+    [[ "$BREAKPOINT" && "$relname" == "$BREAKPOINT" ]] && read -p "Press ENTER to continue ..." 2>&1
 }
 
 # collect scripts given in $1 in the standard subdirectories and
 # sort them by their script file name and
 # source them
-function SourceStage() {
+function SourceStage () {
     stage="$1"
     shift
     STARTSTAGE=$SECONDS
@@ -76,7 +91,7 @@ function SourceStage() {
 }
 
 
-function cleanup_build_area_and_end_program() {
+function cleanup_build_area_and_end_program () {
     # Cleanup build area
     Log "Finished in $((SECONDS-STARTTIME)) seconds"
     if test "$KEEP_BUILD_DIR" ; then
@@ -87,7 +102,6 @@ function cleanup_build_area_and_end_program() {
         rm -Rf $ROOTFS_DIR
         # line below put in comment due to issue #465
         #rm -Rf $BUILD_DIR/outputfs
-        rmdir $v $BUILD_DIR/outputfs >&2
         # in worst case it could not umount; so before remove the BUILD_DIR check if above outputfs is gone
         mount | grep -q "$BUILD_DIR/outputfs"
         if [[ $? -eq 0 ]]; then
@@ -95,9 +109,13 @@ function cleanup_build_area_and_end_program() {
             LogPrint "Directory $BUILD_DIR/outputfs still mounted - trying lazy umount"
             sleep 2
             umount -f -l $BUILD_DIR/outputfs >&2
-            rmdir $v $BUILD_DIR/outputfs >&2
+            rm -Rf $v $BUILD_DIR/outputfs >&2
+        else
+            # not mounted so we can safely delete $BUILD_DIR/outputfs
+            rm -Rf $BUILD_DIR/outputfs
         fi
-        rmdir $v $BUILD_DIR >&2
+        rm -Rf $v $BUILD_DIR >&2
     fi
     Log "End of program reached"
 }
+
