@@ -1,22 +1,9 @@
-# #80_copy_to_tftp.sh
+# 80_copy_to_tftp.sh
 #
 # copy kernel and initrd to TFTP server for Relax-and-Recover
 #
-#    Relax-and-Recover is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation; either version 2 of the License, or
-#    (at your option) any later version.
-
-#    Relax-and-Recover is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
-
-#    You should have received a copy of the GNU General Public License
-#    along with Relax-and-Recover; if not, write to the Free Software
-#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-#
-#
+# This file is part of Relax-and-Recover, licensed under the GNU General
+# Public License. Refer to the included COPYING for full text of license.
 
 # TODO: mount remote TFTP path
 PXE_TFTP_LOCAL_PATH="$PXE_TFTP_PATH"
@@ -24,6 +11,7 @@ PXE_KERNEL="${PXE_TFTP_PREFIX}kernel"
 PXE_INITRD="${PXE_TFTP_PREFIX}initrd.cgz"
 PXE_MESSAGE="${PXE_TFTP_PREFIX}message"
 
+[[ ! -d "$PXE_TFTP_LOCAL_PATH" ]] && mkdir $v -m 750 "$PXE_TFTP_LOCAL_PATH" >&2
 
 cp -pL $v "$KERNEL_FILE" "$PXE_TFTP_LOCAL_PATH/$PXE_KERNEL" >&2
 cp -a $v "$TMP_DIR"/initrd.cgz "$PXE_TFTP_LOCAL_PATH/$PXE_INITRD" >&2
